unit uCalendarClass;

interface

type
  /// represents a calendar with date and some date-methods
  TCalendar = class
    private
      var
        /// actual date
        pDate : TDateTime;
      function getDate : TDateTime;
      function getDay : byte;
      function getMonth : byte;
      function getMonthAsString : String;
      function getYear : integer;
      function getWeekday : byte;
      function getWeekdayAsString : String;
      function getCalendarweek : byte;
    public
      property date : TDateTime read getDate;
      property day : byte read getDay;
      property month : byte read getMonth;
      property monthAsString : String read getMonthAsString;
      property year : integer read getYear;
      property weekday : byte read getWeekday;
      property weekdayAsString : String read getWeekdayAsString;
      property calendarweek : byte read getCalendarweek;
      function isFirstOfMonth : boolean;
      function isLastOfMonth : boolean;
      function dateAsString : String;
      procedure nextDay;
      constructor create (date : TDateTime); overload;
      constructor create (year : integer; month : byte; day : byte); overload;
  end;

implementation

uses
  SysUtils;

{*
  Returns the date

  @return actual date
  @author Pierre Pechloff
*}
function TCalendar.getDate : TDateTime;
begin
  result := pDate;
end;

{*
  Returns the day of month

  @return day of month
  @author Pierre Pechloff
*}
function TCalendar.getDay : byte;
var
  day : word;
  month : word;
  year : word;
begin
  DecodeDate(date,year,month,day);
  result := day;
end;

{*
  Returns the month

  @return month
  @author Pierre Pechloff
*}
function TCalendar.getMonth : byte;
var
  day : word;
  month : word;
  year : word;
begin
  DecodeDate(date,year,month,day);
  result := month;
end;

{*
  Returns the month as text

  @return month as text
  @author Pierre Pechloff
*}
function TCalendar.getMonthAsString : String;
var
  day : word;
  month : word;
  year : word;
begin
  DecodeDate(date,year,month,day);
  case month of
    1 :
      result := 'Januar';
    2 :
      result := 'Februar';
    3 :
      result := 'Mrz';
    4 :
      result := 'April';
    5 :
      result := 'Mai';
    6 :
      result := 'Juni';
    7 :
      result := 'Juli';
    8 :
      result := 'August';
    9 :
      result := 'September';
    10 :
      result := 'Oktober';
    11 :
      result := 'November';
    else
      result := 'Dezember';
  end;
end;

{*
  Returns the year

  @return year
  @author Pierre Pechloff
*}
function TCalendar.getYear : integer;
var
  dummy : word;
  month : word;
  year : word;
begin
  DecodeDate(date,year,month,dummy);
  result := year;
end;

{*
  Returns the weekday

  @return weekday, 0 for monday, 1 for tuesday, ...
  @author Pierre Pechloff
*}
function TCalendar.getWeekday : byte;
begin
  if DayOfWeek(pDate) >= 2 then
    result := DayOfWeek(pDate)-2
  else
    result := DayOfWeek(pDate)+5;
end;

{*
  Returns the weekday as text

  @return weekday as text
  @author Pierre Pechloff
*}
function TCalendar.getWeekdayAsString : String;
begin
  case DayOfWeek(pDate) of
    1 :
      result := 'Sonntag';
    2 :
      result := 'Montag';
    3 :
      result := 'Dienstag';
    4 :
      result := 'Mittwoch';
    5 :
      result := 'Donnerstag';
    6 :
      result := 'Freitag';
    else
      result := 'Samstag';
  end;
end;

{*
  Returns the calendarweek

  @return calendarweek
  @author Pierre Pechloff
*}
function TCalendar.getCalendarweek : byte;
var
  AYear,
  dummy : word;
  First : TDateTime;
begin
  DecodeDate(pDate+((8-DayOfWeek(pDate)) mod 7)-3, AYear, dummy, dummy);
  First := EncodeDate(AYear, 1, 1);
  result := (trunc(pDate-First-3+(DayOfWeek(First)+1) mod 7) div 7)+1
end;

{*
  Returns wether the actual day is the first of a month or not

  @return true if the first day of a month, else false
  @author Pierre Pechloff
*}
function TCalendar.isFirstOfMonth : boolean;
var
  dummy : word;
  tmpMonth : word;
  tmpDay : word;
begin
  DecodeDate(pDate,dummy,tmpMonth,tmpDay);
  result := tmpDay = 1;
end;

{*
  Returns wether the actual day is the last of a month or not

  @return true if the last day of a month, else false
  @author Pierre Pechloff
*}
function TCalendar.isLastOfMonth : boolean;
var
  dummy : word;
  tmpMonth : word;
  tmpDay : word;
begin
  DecodeDate(pDate+1,dummy,tmpMonth,tmpDay);
  result := tmpDay = 1;
end;

{*
  Returns the complete date as text like 'ddd., d. mmmm yyyy'

  @return complete date as text
  @author Pierre Pechloff
*}
function TCalendar.dateAsString : String;
begin
  DateTimeToString(result,'ddd., d. mmmm yyyy',pDate);
end;

{*
  proceeds to the next day

  @author Pierre Pechloff
*}
procedure TCalendar.nextDay;
begin
  pDate := pDate+1;
end;

{*
  Creates a new calendar with the specified date

  @param date to start the calendar with
  @author Pierre Pechloff
*}
constructor TCalendar.create (date : TDateTime);
var
  day : word;
  month : word;
  year : word;
begin
  DecodeDate(date,year,month,day);
  create(year,month,day);
end;

{*
  Creates a new calendar with the specified date

  @param year to start the calendar with
  @param month to start the calendar with
  @param day to start the calendar with
  @author Pierre Pechloff
*}
constructor TCalendar.create (year: Integer; month: Byte; day : byte);
begin
  pDate := EncodeDate(year,month,day);
end;

end.
